@@ -28,6 +28,17 @@
 #include "tracker-miner-manager.h"
 #include "tracker-marshal.h"
 #include "tracker-miner-client.h"
+#include "tracker-miner-dbus.h"
+
+/**
+ * SECTION:tracker-miner-manager
+ * @short_description: External control and monitoring of miners
+ * @include: libtracker-miner/tracker-miner-manager.h
+ *
+ * #TrackerMinerManager keeps track of available miners, their current
+ * progress/status, and also allows basic external control on them, such
+ * as pausing or resuming data processing.
+ **/
 
 #define TRACKER_MINER_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_MINER_MANAGER, TrackerMinerManagerPrivate))
 
@@ -35,6 +46,7 @@
 #define DBUS_NAME_KEY "DBusName"
 #define DBUS_PATH_KEY "DBusPath"
 #define DISPLAY_NAME_KEY "Name"
+#define DESCRIPTION_KEY "Comment"
 
 typedef struct TrackerMinerManagerPrivate TrackerMinerManagerPrivate;
 typedef struct MinerData MinerData;
@@ -43,6 +55,7 @@ struct MinerData {
 	gchar *dbus_name;
 	gchar *dbus_path;
 	gchar *display_name;
+	gchar *description;
 };
 
 struct TrackerMinerManagerPrivate {
@@ -77,6 +90,16 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 
 	object_class->finalize = miner_manager_finalize;
 
+	/**
+	 * TrackerMinerManager::miner-progress
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 * @status: miner status
+	 * @progress: miner progress, from 0 to 1
+	 *
+	 * The ::miner-progress signal is meant to report status/progress changes
+	 * in any tracked miner.
+	 **/
 	signals [MINER_PROGRESS] =
 		g_signal_new ("miner-progress",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -88,6 +111,14 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 			      G_TYPE_STRING,
 			      G_TYPE_STRING,
 			      G_TYPE_DOUBLE);
+	/**
+	 * TrackerMinerManager::miner-paused
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 *
+	 * The ::miner-paused signal will be emitted whenever a miner
+	 * (referenced by @miner) is paused.
+	 **/
 	signals [MINER_PAUSED] =
 		g_signal_new ("miner-paused",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -97,6 +128,14 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
+	/**
+	 * TrackerMinerManager::miner-resumed
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 *
+	 * The ::miner-resumed signal will be emitted whenever a miner
+	 * (referenced by @miner) is resumed.
+	 **/
 	signals [MINER_RESUMED] =
 		g_signal_new ("miner-resumed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -106,6 +145,15 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
+	/**
+	 * TrackerMinerManager::miner-activated
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 *
+	 * The ::miner-activated signal will be emitted whenever a miner
+	 * (referenced by @miner) is activated (technically, this means
+	 * the miner has appeared in the session bus).
+	 **/
 	signals [MINER_ACTIVATED] =
 		g_signal_new ("miner-activated",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -115,6 +163,15 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
+	/**
+	 * TrackerMinerManager::miner-deactivated
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 *
+	 * The ::miner-deactivated signal will be emitted whenever a miner
+	 * (referenced by @miner) is deactivated (technically, this means
+	 * the miner has disappeared from the session bus).
+	 **/
 	signals [MINER_DEACTIVATED] =
 		g_signal_new ("miner-deactivated",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -331,12 +388,28 @@ miner_manager_finalize (GObject *object)
 	G_OBJECT_CLASS (tracker_miner_manager_parent_class)->finalize (object);
 }
 
+/**
+ * tracker_miner_manager_new:
+ *
+ * Creates a new #TrackerMinerManager instance.
+ *
+ * Returns: a #TrackerMinerManager.
+ **/
 TrackerMinerManager *
 tracker_miner_manager_new (void)
 {
 	return g_object_new (TRACKER_TYPE_MINER_MANAGER, NULL);
 }
 
+/**
+ * tracker_miner_manager_get_running:
+ * @manager: a #trackerMinerManager
+ *
+ * Returns a list of references for all active miners.
+ *
+ * Returns: a #GSList of miner references. This list must be freed
+ *          through g_slist_free(), and all contained data with g_free().
+ **/
 GSList *
 tracker_miner_manager_get_running (TrackerMinerManager *manager)
 {
@@ -386,7 +459,7 @@ crawler_check_file_cb (TrackerCrawler *crawler,
 	TrackerMinerManager *manager;
 	TrackerMinerManagerPrivate *priv;
 	GKeyFile *key_file;
-	gchar *path, *dbus_path, *dbus_name, *display_name;
+	gchar *path, *dbus_path, *dbus_name, *display_name, *description;
 	GError *error = NULL;
 	MinerData *data;
 
@@ -414,7 +487,7 @@ crawler_check_file_cb (TrackerCrawler *crawler,
 	display_name = g_key_file_get_locale_string (key_file, DESKTOP_ENTRY_GROUP, DISPLAY_NAME_KEY, NULL, NULL);
 
 	if (!dbus_path || !dbus_name || !display_name) {
-		g_warning ("Essential data (DBusPath or Name) are missing in miner .desktop file");
+		g_warning ("Essential data (DBusPath, DBusName or Name) are missing in miner .desktop file");
 		g_key_file_free (key_file);
 		g_free (dbus_path);
 		g_free (display_name);
@@ -423,10 +496,13 @@ crawler_check_file_cb (TrackerCrawler *crawler,
 		return FALSE;
 	}
 
+	description = g_key_file_get_locale_string (key_file, DESKTOP_ENTRY_GROUP, DESCRIPTION_KEY, NULL, NULL);
+
 	data = g_slice_new0 (MinerData);
 	data->dbus_path = dbus_path;
 	data->dbus_name = dbus_name;
 	data->display_name = display_name;
+	data->description = description;
 
 	priv->miners = g_list_prepend (priv->miners, data);
 
@@ -475,6 +551,15 @@ initialize_miners_data (TrackerMinerManager *manager)
 	g_object_unref (crawler);
 }
 
+/**
+ * tracker_miner_manager_get_available:
+ * @manager: a #TrackerMinerManager
+ *
+ * Returns a list of references for all available miners.
+ *
+ * Returns: a #GSList of miner references. This list must be freed
+ *          through g_slist_free(), and all contained data with g_free().
+ **/
 GSList *
 tracker_miner_manager_get_available (TrackerMinerManager *manager)
 {
@@ -493,6 +578,19 @@ tracker_miner_manager_get_available (TrackerMinerManager *manager)
 	return g_slist_reverse (list);
 }
 
+/**
+ * tracker_miner_manager_pause:
+ * @manager: a #TrackerMinerManager.
+ * @miner: miner reference
+ * @reason: reason to pause
+ * @cookie: return location for the pause cookie ID
+ *
+ * Asks @miner to pause. a miner could be paused by
+ * several reasons, and its activity won't be resumed
+ * until all pause requests have been resumed.
+ *
+ * Returns: %TRUE if the miner was paused successfully.
+ **/
 gboolean
 tracker_miner_manager_pause (TrackerMinerManager *manager,
 			     const gchar         *miner,
@@ -536,6 +634,17 @@ tracker_miner_manager_pause (TrackerMinerManager *manager,
 	return TRUE;
 }
 
+/**
+ * tracker_miner_manager_resume:
+ * @manager: a #TrackerMinerManager
+ * @miner: miner reference
+ * @cookie: pause cookie
+ *
+ * Tells @miner to resume activity. The miner won't actually resume
+ * operations until all pause requests have been resumed.
+ *
+ * Returns: %TRUE if the miner was successfully resumed.
+ **/
 gboolean
 tracker_miner_manager_resume (TrackerMinerManager *manager,
 			      const gchar         *miner,
@@ -564,6 +673,15 @@ tracker_miner_manager_resume (TrackerMinerManager *manager,
 	return TRUE;
 }
 
+/**
+ * tracker_miner_manager_is_active:
+ * @manager: a #TrackerMinerManager
+ * @miner: miner reference
+ *
+ * Returns %TRUE if @miner is currently active.
+ *
+ * Returns: %TRUE if @miner is active.
+ **/
 gboolean
 tracker_miner_manager_is_active (TrackerMinerManager *manager,
 				 const gchar         *miner)
@@ -591,6 +709,151 @@ tracker_miner_manager_is_active (TrackerMinerManager *manager,
 	return active;
 }
 
+/**
+ * tracker_miner_manager_get_status:
+ * @manager: a #TrackerMinerManager
+ * @miner: miner reference
+ * @status: return location for status
+ * @progress: return location for progress
+ *
+ * Returns the current status and progress for @miner.
+ *
+ * Returns: %TRUE if the status could be retrieved successfully.
+ **/
+gboolean
+tracker_miner_manager_get_status (TrackerMinerManager  *manager,
+				  const gchar          *miner,
+				  gchar               **status,
+				  gdouble              *progress)
+{
+	DBusGProxy *proxy;
+	GError *error = NULL;
+	gdouble p;
+	gchar *st;
+
+	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), FALSE);
+	g_return_val_if_fail (miner != NULL, FALSE);
+
+	proxy = find_miner_proxy (manager, miner);
+
+	if (!proxy) {
+		g_critical ("No D-Bus proxy found for miner '%s'", miner);
+		return FALSE;
+	}
+
+	org_freedesktop_Tracker1_Miner_get_progress (proxy, &p, &error);
+
+	if (error) {
+		g_critical ("Could not get miner progress for '%s': %s", miner,
+			    error->message);
+		g_error_free (error);
+
+		return FALSE;
+	}
+
+	org_freedesktop_Tracker1_Miner_get_status (proxy, &st, &error);
+
+	if (error) {
+		g_critical ("Could not get miner status for '%s': %s", miner,
+			    error->message);
+		g_error_free (error);
+
+		return FALSE;
+	}
+
+	if (status) {
+		*status = st;
+	} else {
+		g_free (st);
+	}
+
+	if (progress) {
+		*progress = p;
+	}
+
+	return TRUE;
+}
+
+/**
+ * tracker_miner_manager_is_paused:
+ * @manager: a #TrackerMinerManager
+ * @miner: miner reference
+ * @applications: return location for application names.
+ * @reasons: return location for pause reasons.
+ *
+ * This function either returns %FALSE if the miner is not paused,
+ * or returns %TRUE and fills in @applications and @reasons with
+ * the pause reasons and the applications that asked for it. Both
+ * arrays will have the same lengh, and will be sorted so the
+ * application/pause reason pairs have the same index.
+ *
+ * Returns: %TRUE if @miner is paused.
+ **/
+gboolean
+tracker_miner_manager_is_paused (TrackerMinerManager *manager,
+				 const gchar         *miner,
+				 GStrv               *applications,
+				 GStrv               *reasons)
+{
+	DBusGProxy *proxy;
+	GStrv apps, r;
+	GError *error = NULL;
+	gboolean paused;
+
+	if (applications) {
+		*applications = NULL;
+	}
+
+	if (reasons) {
+		*reasons = NULL;
+	}
+
+	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), TRUE);
+	g_return_val_if_fail (miner != NULL, TRUE);
+
+	proxy = find_miner_proxy (manager, miner);
+
+	if (!proxy) {
+		g_critical ("No D-Bus proxy found for miner '%s'", miner);
+		return FALSE;
+	}
+
+	org_freedesktop_Tracker1_Miner_get_pause_details (proxy, &apps, &r, &error);
+
+	if (error) {
+		g_critical ("Could not get pause details for miner '%s': %s", miner,
+			    error->message);
+		g_error_free (error);
+
+		return TRUE;
+	}
+
+	paused = (g_strv_length (apps) > 0);
+
+	if (applications) {
+		*applications = apps;
+	} else {
+		g_strfreev (apps);
+	}
+
+	if (reasons) {
+		*reasons = r;
+	} else  {
+		g_strfreev (r);
+	}
+
+	return paused;
+}
+
+/**
+ * tracker_miner_manager_get_display_name:
+ * @manager: a #TrackerMinerManager
+ * @miner: miner reference
+ *
+ * Returns a translated display name for @miner.
+ *
+ * Returns: The miner display name.
+ **/
 const gchar *
 tracker_miner_manager_get_display_name (TrackerMinerManager *manager,
 					const gchar         *miner)
@@ -613,3 +876,35 @@ tracker_miner_manager_get_display_name (TrackerMinerManager *manager,
 
 	return NULL;
 }
+
+/**
+ * tracker_miner_manager_get_description:
+ * @manager: a #TrackerMinerManager
+ * @miner: miner reference
+ *
+ * Returns the description for @miner, or %NULL if none is specified.
+ *
+ * Returns: The miner description.
+ **/
+const gchar *
+tracker_miner_manager_get_description (TrackerMinerManager *manager,
+				       const gchar         *miner)
+{
+	TrackerMinerManagerPrivate *priv;
+	GList *m;
+
+	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), NULL);
+	g_return_val_if_fail (miner != NULL, NULL);
+
+	priv = TRACKER_MINER_MANAGER_GET_PRIVATE (manager);
+
+	for (m = priv->miners; m; m = m->next) {
+		MinerData *data = m->data;
+
+		if (strcmp (miner, data->dbus_name) == 0) {
+			return data->description;
+		}
+	}
+
+	return NULL;
+}
